class Error(Exception):
    pass

class DiscordAPIError(Error):
    """
        If there is a Discord API Error this will be raised
    """
    def __init__(self, status, message):
        self.status = status
        self.message = message
        super().__init__(self.message)
        
    def __str__(self):
        return f'{self.status}: {self.message}'

class WebsocketError(Error):
    """
        Websocket Error handling
    """
    def __init__(self, status, message):
        self.status = status
        self.message = message
        super().__init__(self.message)
        
    def __str__(self):
        return f'{self.status}: {self.message}'


class Intents:
    GUILDS = 1 << 0
    GUILD_MEMBERS = 1 << 1
    GUILD_BANS = 1 << 2
    GUILD_EMOJIS_AND_STICKERS = 1 << 3
    GUILD_INTEGRATIONS = 1 << 4
    GUILD_WEBHOOKS = 1 << 5
    GUILD_INVITES = 1 << 6
    GUILD_VOICE_STATES = 1 << 7
    GUILD_PRESENCES = 1 << 8
    GUILD_MESSAGES = 1 << 9
    GUILD_MESSAGE_REACTIONS = 1 << 10
    GUILD_MESSAGE_TYPING = 1 << 11
    DIRECT_MESSAGES = 1 << 12
    DIRECT_MESSAGE_REACTIONS = 1 << 13
    DIRECT_MESSAGE_TYPING = 1 << 14


import requests

class RESTManager:
    def __init__(self, rest, endpoint):
        self.token = rest;
        
        self.headers = {
            'Authorization': f'Bot {rest}',
            'Content-Type': 'application/json'
        }
        
    def RESTGetCurrentUser(self, rest, endpoint):
        
        r = requests.get(f'{endpoint}/users/@me', headers=self.headers)
        
        return r
        
    def RESTPostMessage(self, rest, endpoint, channelId, json):
        
        self.MESSAGE_JSON = json
        
        r = requests.post(f'{endpoint}/channels/{channelId}/messages', headers=self.headers, json=self.MESSAGE_JSON)
        
        return r
        
    def RESTPost(self, endpoint, headers, json):
        
        self.postHeaders = headers
        self.postJSON = json
        
        post = requests.post(f'{endpoint}', headers=self.postHeaders, json=self.postJSON)
        
        return post 
        
    def RESTGet(self, endpoint, headers):
        self.getHeaders = headers
        
        get = requests.get(endpoint, headers=self.getHeaders)
        
        return get


import asyncio
import pymitter
import json

class ClientUser:
    def __init__(self, data):
        x = json.dumps(data.json())
        usr_data = json.loads(x)['d']
        
        self.id = usr_data['user']['id']
        
        self.discriminator = usr_data['user']['discriminator']
        
        self.username = usr_data['user']['username']

        self.tag = f'{self.username}#{self.discriminator}'


import aiohttp
import asyncio
import json
import platform
import random

APIEndpoint = 'https://discord.com/api/v9'

class WebsocketManager:
    """
        Websocket Manager for Discord's WebSocket
    """
    def __init__(self, data):
        self.token = data.token
        self.client = data
        self.data = data

    async def connect(self, connection):
        self.connection = connection
        
        self.identifyBoilerplate = {}
        self.identifyBoilerplate['d'] = {}
        self.identifyBoilerplate['d']['token'] = self.token
                    
        # Indeity Opcode 
        self.identifyBoilerplate['op'] = 2
                    
        # Properties
        self.identifyBoilerplate['d']['properties'] = {
                    '$os': str(platform.system()).lower(),
                    '$browser': 'gydo.py',
                    '$device': 'gydo.py'
                }
                    
        # Intents
        self.identifyBoilerplate['d']['intents'] = 513
                
        await self.connection.send_json(self.identifyBoilerplate)
        
        opHello = await self.connection.receive()
        ack = await self.connection.receive()
        
        usr = await self.connection.receive()
        
        dump = json.dumps(usr.json())
        res = json.loads(dump)
        
        return usr

    async def sendMessage(self, message):
        await self.connection.send_str(message)
        
    async def rcv_messages(self):
        while True:
            try:
                x = await self.connection.receive()
                print(x)
            except KeyboardInterrupt:
                await self.connection.close()

    async def heartbeat(self):
        while True:
            try:
                await self.connection.send_str(json.dumps({"op":1,'d':2}))
                await asyncio.sleep(4.1250 * random.random())
            except KeyboardInterrupt:
                await self.connection.close()
    
    async def set_status(self, name):
        self.identifyBoilerplate['d']['presence'] = {}
        self.identifyBoilerplate['d']['presence']['status'] = name
        
        return


from setuptools import setup, find_packages

VERSION = '0.0.31'
DESCRIPTION = 'A Package for interacting with Discord\'s API'

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="gydo.py",
    version=VERSION,
    author="loldonut (John Heinrich)",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="Apache-2.0",
    packages=find_packages(),
    install_requires=['requests', 'schedule', 'websockets', 'aiohttp', 'nest_asyncio', 'pymitter'],
    keywords=['python', 'discord', 'gydo.py', 'discord api']
)


from restapi import RESTManager
from WebsocketManager import WebsocketManager
import json
from ClientUser import ClientUser
import asyncio
import aiohttp
import nest_asyncio
nest_asyncio.apply()
from pymitter import EventEmitter

APIEndpoint = 'https://discord.com/api/v9'


class Client:
    """
        Main Client for Discord Bots
    """
    def __init__(self, token):
        self.endpoint = APIEndpoint
        self.token = token
        self.ws = WebsocketManager(self)
        self.manage = RESTManager(self.token, APIEndpoint)
        self.events = EventEmitter()
        
        asyncio.run(self.create_session())
    
    """
        Send a message through Discord's REST API
    """
    async def sendMessage(self, message, channelId, *embed):
        self.messageJSON = {
            'content': f'{message}',
            'embeds': []
        }

        r = self.manage.RESTPostMessage(self.token, APIEndpoint, channelId, self.messageJSON)
        
        return r

    async def MessageEmbed(self, title, description):
        self.MessageEmbedJSON = {
            'title': title,
            'description': description
        }

        return self.MessageEmbedJSON
        
    """
        Create a WebSocket Session
    """
    async def create_session(self):
        loop = asyncio.get_event_loop()
        
        self.session = aiohttp.ClientSession()

        self.connection = await self.session.ws_connect('wss://gateway.discord.gg/?v=9&encoding=json')
        
        tasks = [
            asyncio.ensure_future(self.ws.heartbeat()),
        ]
        
        self.user = ClientUser(await self.ws.connect(self.connection))
        
        try:
            loop.run_until_complete(asyncio.wait(tasks))
            loop.run_forever()
        except KeyboardInterrupt:
            await self.ws.connection.close()